package vazkii.botania.common.item.equipment.bauble;

import baubles.api.BaubleType;
import baubles.api.BaublesApi;
import com.google.common.base.Predicates;
import com.google.common.util.concurrent.ListenableFutureTask;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.monster.EntityCreeper;
import net.minecraft.entity.monster.IMob;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import vazkii.botania.api.item.IBaubleRender;
import vazkii.botania.api.mana.IManaUsingItem;
import vazkii.botania.api.mana.ManaItemHandler;
import vazkii.botania.common.block.subtile.functional.SubTileHeiseiDream;
import vazkii.botania.common.core.handler.ModSounds;
import vazkii.botania.common.network.PacketBotaniaEffect;
import vazkii.botania.common.network.PacketHandler;

import java.util.List;

public class ItemDivaCharm extends ItemBauble implements IManaUsingItem, IBaubleRender {
    public ItemDivaCharm() {
        super("divaCharm");
        MinecraftForge.EVENT_BUS.register(this);
    }

    @SubscribeEvent
    public void onEntityDamaged(LivingHurtEvent event) {
        EntityLivingBase eb = event
                .getEntityLiving();
        if (event.getSource().getImmediateSource() instanceof EntityPlayer && eb instanceof EntityLiving &&
                !eb.world.isRemote && eb.isNonBoss() &&
                Math.random() < 0.6D) {
            EntityLiving target = (EntityLiving) eb;
            EntityPlayer player = (EntityPlayer) event.getSource().getImmediateSource();
            ItemStack amulet = BaublesApi.getBaublesHandler(player).getStackInSlot(6);
            if (!amulet.isEmpty() && amulet.getItem() == this) {
                Runnable lambda = () -> {
                    if (ManaItemHandler.requestManaExact(amulet, player, 250, false)) {
                        List<EntityLiving> mobs = player.world.getEntitiesWithinAABB(EntityLiving.class,
                                                                               new AxisAlignedBB(target.posX - 20.0D,
                                                                                                 target.posY - 20.0D,
                                                                                                 target.posZ - 20.0D,
                                                                                                 target.posX + 20.0D,
                                                                                                 target.posY + 20.0D,
                                                                                                 target.posZ + 20.0D),
                                                                               Predicates.instanceOf(IMob.class));
                        if (mobs.size() > 1 && SubTileHeiseiDream.brainwashEntity(target, mobs)) {
                            target.heal(target.getMaxHealth());
                            target.isDead = false;
                            if (target instanceof EntityCreeper) {
                                ((EntityCreeper) target).timeSinceIgnited = 2;
                            }
                            ManaItemHandler.requestManaExact(amulet, player, 250, true);
                            player.world.playSound(null, player.posX, player.posY, player.posZ, ModSounds.divaCharm,
                                                   SoundCategory.PLAYERS, 1.0F, 1.0F);
                            PacketHandler.sendToNearby(target.world, target,
                                                       new PacketBotaniaEffect(
                                                               PacketBotaniaEffect.EffectType.DIVA_EFFECT, target.posX,
                                                               target.posY, target.posZ,
                                                               target.getEntityId()));
                        }
                    }
                };
                MinecraftServer server = FMLCommonHandler.instance().getMinecraftServerInstance();
                synchronized (server.futureTaskQueue) {
                    server.futureTaskQueue.add(ListenableFutureTask.create(lambda, null));
                }
            }
        }
    }

    public BaubleType getBaubleType(ItemStack arg0) { return BaubleType.CHARM; }

    public boolean usesMana(ItemStack stack) { return true; }

    @SideOnly(Side.CLIENT)
    public void onPlayerBaubleRender(ItemStack stack, EntityPlayer player, IBaubleRender.RenderType type, float partialTicks) {
        if (type == IBaubleRender.RenderType.HEAD) {
            IBaubleRender.Helper.translateToHeadLevel(player);
            (Minecraft.getMinecraft()).renderEngine.bindTexture(TextureMap.LOCATION_BLOCKS_TEXTURE);
            GlStateManager.scale(0.8D, 0.8D, 0.8D);
            GlStateManager.rotate(-90.0F, 0.0F, 1.0F, 0.0F);
            GlStateManager.rotate(180.0F, 1.0F, 0.0F, 0.0F);
            GlStateManager.translate(0.1625D, -1.625D, 0.4D);
            Minecraft.getMinecraft()
                     .getRenderItem()
                     .renderItem(new ItemStack(this, 1), ItemCameraTransforms.TransformType.GROUND);
        }
    }
}
