package vazkii.botania.common.block.subtile.functional;

import com.google.common.base.Predicates;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.ai.EntityAITasks;
import net.minecraft.entity.monster.IMob;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import vazkii.botania.api.lexicon.LexiconEntry;
import vazkii.botania.api.subtile.RadiusDescriptor;
import vazkii.botania.api.subtile.SubTileFunctional;
import vazkii.botania.common.lexicon.LexiconData;

import java.util.List;

public class SubTileHeiseiDream extends SubTileFunctional {
    private static final int RANGE = 5;

    private AxisAlignedBB box;
    private BlockPos      cp;

    public void onUpdate() {
        super.onUpdate();
        if (supertile.getWorld().isRemote) {
            return;
        }
        if (!supertile.getPos().equals(cp)) {
            box = new AxisAlignedBB(supertile.getPos().add(-5, -5, -5),
                                    supertile.getPos().add(6, 6, 6));
            cp = supertile.getPos();
        }
        List<EntityLiving> mobs = supertile.getWorld()
                                           .getEntitiesWithinAABB(EntityLiving.class, box,
                                                                  Predicates.instanceOf(IMob.class));
        if (mobs.size() > 1 && this.mana >= 100) {
            for (EntityLiving mob : mobs) {
                if (brainwashEntity(mob, mobs)) {
                    this.mana -= 100;
                    sync();
                    break;
                }
            }
        }
    }

    public static boolean brainwashEntity(EntityLiving entity, List<EntityLiving> mobs) {
        EntityLivingBase target = entity.getAttackTarget();
        if (!(target instanceof IMob)) {
            int i = mobs.indexOf(entity);
            if (i != -1) {
                if (mobs.size() == 1) {
                    return false;
                }
                if (i == (i = entity.world.rand.nextInt(mobs.size() - 1))) {
                    i = mobs.size() - 1;
                }
            } else {
                i = entity.world.rand.nextInt(mobs.size());
            }
            entity.setAttackTarget(null);
            for (EntityAITasks.EntityAITaskEntry entry : entity.targetTasks.taskEntries) {
                if (entry.action instanceof net.minecraft.entity.ai.EntityAIHurtByTarget) {
                    entity.targetTasks.removeTask(entry.action);
                    entity.targetTasks.addTask(-1, entry.action);
                    break;
                }
            }
            entity.setRevengeTarget(mobs.get(i));
            return true;
        }
        return false;
    }

    public RadiusDescriptor getRadius() { return new RadiusDescriptor.Square(toBlockPos(), 5); }

    public int getColor() { return 16720285; }

    public int getMaxMana() { return 1000; }

    public LexiconEntry getEntry() { return LexiconData.heiseiDream; }
}
